/*
 * (c) Copyright 1999-2014 PaperCut Software International Pty. Ltd.
 *
 * MFD related utilities
 */
"use strict";
var MFD = function () {
  var m_currentScreen = null;
  var m_message = null;
  var m_deviceStatus = null;
  var m_userAuthInfo = null;
  var m_timerId = null;
  var serverCheckTimer = null;
  var baseUrl = null;
  var connectionCheckTimeout = null;
  var deviceVendor = null;
  var deviceModel = null;
  var deviceGeneration = null;
  var ewbVersion = null;

  // Whether currently the server is online or offline
  var offline = true;
  // How often to recheck while offline
  var offlineCheck = 10000;
  // How often to recheck while online
  var onlineCheck = 60000;

  function clearTimer() {
    if (m_timerId !== null) {
      clearTimeout(m_timerId);
      m_timerId = null;
    }
  }

  function setScreen(currentScreen, newScreen) {
    status("setScreen: " + newScreen);
    var div;
    if (currentScreen) {
      div = document.getElementById(currentScreen);
      div.style.display = "none";
    }
    div = document.getElementById(newScreen);
    div.style.display = "inline";
    window.focus();

    // transition to new screen
    status("invoke screen handler: " + newScreen + "_function");
    eval(newScreen + "_function();");
  }

  function loadURL(url) {
    // May need to disable built in EWB error screen control:
    // via SetScreenChange('onConnectionError:CONTINUE')
    // see Apeos iiX Custom Service Script External Specification 6.13.8
    var frame = document.getElementById('frame');

    // offline -> online: always reinitialize UI to start over
    frame.style.display = 'inline';
    frame.src = baseUrl + url + "?time=" + new Date().getTime();
    frame.onload = function() {
      // sync the page title from the embedded UI page once loaded
      document.title = this.contentDocument.title;
    };
    // Resize iframe to full-screen
    if (!debugEnabled) {
      if (typeof window.innerHeight !== 'undefined') {
        frame.style.height = window.innerHeight + 'px';
      }
    }
    MFD.logEvent("Change browser URL:" + frame.src);
    frame = null;
  }

  function onServerCheckDone(url, resp) {
    if (resp.results.result !== 'true') {
      // URL not accessible, pass through the plugin message
      onServerCheckFail(url, true, resp.results.msg);
    } else {
      // URL is accessible, initialise the UI
      DIALOG.closeMessageDialog();

      if (offline) {
        loadURL(url);
      }

      // Keep checking the status periodically using online timeout setting
      offline = false;
      serverCheckTimer && clearTimeout(serverCheckTimer);
      serverCheckTimer = setTimeout(function() { MFD.checkServerStatus(url); }, onlineCheck);
    }
  }

  function onServerCheckFail(url, connectionChecked, message) {
    if (debugEnabled) {
      DIALOG.closeMessageDialog();

      // TODO: static papercut screen/background to prevent empty screen, may do it with a dynamic generation at setup time
      // TODO localize
      DIALOG.showMessageDialog(
          'Connection check to ' + document.title +
          ' (' + getHostname(url) + ') failed.\nResult: ' + message);
    }

    // online -> offline
    // Schedule a retry of connection check using offline timeout setting
    offline = true;

    var retryConnection = function(timeout) {
      serverCheckTimer && clearTimeout(serverCheckTimer);
      serverCheckTimer = setTimeout(function() { MFD.checkServerStatus(url); }, timeout || offlineCheck);
    };

    if (connectionChecked) {
      // Connection check ran successfully and found server offline, keep retrying
      retryConnection();
    } else {
      // Connection check itself failed, this is a fatal error, retry after user acknowledges
      DIALOG.showMessageConfirm(message, function () {
        retryConnection(10);
      });
    }
  }

  function getHostname(url) {
    var parser = document.createElement('a');
    parser.href = url;
    return parser.hostname;
  }

  function mapXCPErrorMessage(message) {
    if (message && /1010-iccard setting is disabled/.test(message)) {
      // Card reader is not enabled on device
      // TODO l18n
      return "Incomplete configuration: the card reader is not enabled on this device.\n" +
        "Please set Security > Smart Card Settings in the administrative interface of this device.\n\n" +
        message;
    } else if (message && /1012-request is not available/.test(message)) {
      // Unable to communicate with XCP on the device
      return "Incomplete configuration: Unable to communicate with the device.\n" +
        "Please set Security > Authentication Configuration > Login Type in the administrative interface of this device.\n\n" +
        message;
    } else {
      return message;
    }
  }

  return {
    /**
     * Initialise this module using provided base URL and connection timeout settings
     */
    init: function(config, onSuccess, onError) {
      if (!config || !config.url) {
        DIALOG.showMessageConfirm("Incomplete configuration: server URL is not set, please re-configure this device",
            function () { MFD.init(config, onSuccess, onError); }
          );
        return;
      }

      connectionCheckTimeout = config.connectionTimeout;
      baseUrl = config.url;
      deviceVendor = config.deviceVendor;
      deviceModel = config.deviceModel;
      deviceGeneration = config.deviceGeneration;
      ewbVersion = config.browserVersion;

      if (config.skipDeviceReady) {
         // If requested to skip device ready check then invoke the callback and exit
         MFD.logEvent("Skip device ready check");
         onSuccess && onSuccess();
         return;
      }

      // Send a "pilot" XCP request through to check the status of XCP sub system
      // On DMP9 machines custom service can start before XCP is ready and results in
      // "1199-xcpauth internal error" returned until the device is ready.
      DIALOG.showMessageDialog('Please wait ...', {COLOR : 'black'});

      MFD.checkDeviceReady(
          function(resp) {
            // Continue startup on success but clear the timer if we kicked one off during init
            clearTimer();
            onSuccess && onSuccess(resp);
          },
          function(message) {
            if (message && !/1199-xcpauth internal error/.test(message)) {
              // unexpected error fire caller's callback, 1199 may occur on startup so be quiet for that one.
              onError && onError(message);
            }

            // Retry initialisation after error
            clearTimer();
            m_timerId = setTimeout(function() {
              MFD.init(config, onSuccess, onError);
            }, 5000);
          });
    },

    getScreen: function() {
      return m_currentScreen;
    },

    getMessage: function() {
      return m_message;
    },

    getDeviceStatus: function() {
      return m_deviceStatus;
    },

    getUserInfo: function() {
      return m_userAuthInfo;
    },

    /**
     * Change screen to destination screen name using screenFunction as callback
     */
    changeScreen: function(screen, screenFunction) {
      status("changeScreen: " + screen);
      MFD.logEvent("changeScreen:" + screen);
      clearTimer();

      // invoke screen callback in the context of the old screen
      screenFunction = screenFunction || setScreen;
      screenFunction(m_currentScreen, screen);
      m_currentScreen = screen;
    },

    /**
     *  Automatically change screen after a timeout
     */
    autoChange: function(time, screen, screenFunction) {
      clearTimer();
      m_timerId = setTimeout(function() {
        AUTH.changeScreen(screen, screenFunction);
      }, time);
    },

    /**
     * Verify the URL is online via a custom XCP action
     */
    checkServerConnection : function(url, onSuccess, onError) {
      if (connectionCheckEnabled) {
        var param = "method=checkServerConnection" + "&URL=" + encodeURIComponent(url);
        if (connectionCheckTimeout) {
          param += "&timeout=" + connectionCheckTimeout;
        }
        // Note: using custom action for connection check as doing it directly
        // in the browser via XHR results in 'SECURITY_ERR(00010012)' on EWB
        MFD.doAction(param, onSuccess, onError);
      } else {
        onSuccess({results: {result:'true'}});
      }
    },

    /**
     * Fetch server configuration by the embedded plugins
     */
    getServerConfiguration : function(url, onSuccess, onError) {
      var param = "method=getServerConfiguration" + "&URL=" + encodeURIComponent(url);
      if (connectionCheckTimeout) {
        param += "&timeout=" + connectionCheckTimeout;
      }
      // Note: using custom action for as doing it directly
      // in the browser via XHR results in 'SECURITY_ERR(00010012)' on EWB
      MFD.doAction(param, onSuccess, onError);
    },

    /**
     * Check device (XCP) is ready
     */
    checkDeviceReady : function(onSuccess, onError) {
      var param = "method=checkDeviceReady"
        + "&deviceModel=" + encodeURIComponent(deviceModel)
        + "&ewbVersion=" + encodeURIComponent(ewbVersion);

      MFD.doAction(param,
        function(resp) {
          onSuccess && onSuccess(resp);
        },
        function(message) {
          onError && onError(message);
        });
    },

    /**
     * Log local UI event
     */
    logEvent : function(event, onSuccess, onError) {
      var param = "method=logEvent" + "&Event=" + encodeURIComponent(event);
      MFD.doAction(param,
        function(resp) {
          onSuccess && onSuccess(resp);
        },
        function(message) {
          onError && onError(message);
        });
    },

    /**
     * Retrieve currently logged user (if any)
     */
    getLoggedOnUser : function(onSuccess, onError) {
      var param = "method=getLoggedInUserAction";
      MFD.doAction(param,
        function(resp) {
          onSuccess && onSuccess(resp);
        },
        function(message) {
          onError && onError(message);
        });
    },

    /**
     * Cache last known card number as an "in-progress" card login.
     */
    setLastCardNumber : function(cardNumber, onSuccess, onError) {
      var param = "method=setLastCardNumber" + "&cardNumber=" + cardNumber;
      MFD.doAction(param,
        function(resp) {
          onSuccess && onSuccess(resp);
        },
        function(message) {
          onError && onError(message);
        });
    },

    /**
     * Reset any in-progress login state (ie. soft logout vs full logout of the device user).
     */
    loginReset : function(onSuccess, onError) {
      var param = "method=loginReset";
      MFD.doAction(param,
        function(resp) {
          onSuccess && onSuccess(resp);
        },
        function(message) {
          onError && onError(message);
        });
    },

    /**
     * Check application server URL is online
     */
    checkServerStatus : function(url, onSuccess, onError) {
      // Check if the configured server is online and hosts the device URL
      if (offline && connectionCheckEnabled) {
        // only show notice while offline
        // TODO localize
        if (debugEnabled) {
          DIALOG.showMessageDialog('Connecting to server: ' + getHostname(url) + ' ...', {COLOR : 'black'});
        } else {
          DIALOG.showMessageDialog('Connecting to server ...', {COLOR : 'black'});
        }
      }
      // Perform online check on a connection check REST service
      MFD.checkServerConnection(baseUrl + '/init?checkConnection=true',
        function(resp) {
          onServerCheckDone(url, resp);
          onSuccess && onSuccess(resp);
        },
        function(message) {
          onServerCheckFail(url, false, message);
          onError && onError(message);
        });
    },

    /**
     * Authenticate using parameters
     */
    authenticate: function(param, onSuccess, onError) {
      XCP.callService("authenticate", param,
        function(result) {
          m_userAuthInfo = result;
          if (onSuccess) {
            onSuccess(result);
          }
        },
        function(message) {
          m_userAuthInfo = null;
          message = mapXCPErrorMessage(message);
          m_message = message;
          if (onError) {
            onError(message);
          }
        });
    },

    /**
     * Logout underlying user
     */
    logout: function(onSuccess, onError) {
      XCP.callService("logout", null,
        function(result) {
          onSuccess && onSuccess(result);
        },
        function(message) {
          onError && onError(message);
        });
    },

    /**
     * Lookup current card reader status
     */
    getStatus: function(onSuccess, onError) {
      XCP.callService("getstatus", null,
        function(resp) {
          m_deviceStatus = resp;
          onSuccess && onSuccess(resp);
        },
        function(message) {
          m_deviceStatus = null;
          message = mapXCPErrorMessage(message);
          m_message = message;
          onError && onError(message);
        });
    },

    /**
     * Invoke generic XCP action
     */
    doAction: function(param, onSuccess, onError) {
      XCP.callService("doAction", param,
        function(resp) {
          onSuccess && onSuccess(resp);
        },
        function(message) {
          message = mapXCPErrorMessage(message);
          m_message = message;
          onError && onError(message);
        });
    },

    /**
     * Map specified destination screen name to a valid destination name.
     */
    mapScreenName: function(screenName) {
      if (!screenName) {
        return "allservice";
      }

      switch (screenName) {
        case "menu":
        case "allservice":    /* all services home */
        case "copy":          /* copy */
        case "copy_easy":     /* simple copy */
        case "box":           /* extended folder */
        case "scan_mbox":     /* scan to folder */
        case "scan_email":    /* scan to email */
        case "scan_it":       /* Job template */
        case "scan_pc":       /* scan to pc */
        case "scan_url":      /* store & send link */
        case "scan_media":    /* store to USB */
        case "scan_wsd":      /* scan (WSD) */
        case "fax":           /* fax */
        case "sfax":          /* Server fax */
        case "ifax":          /* Internet fax */
        case "fax_easy":      /* Simple Fax */
        case "mp_doc":        /* Media print - text */
        case "mp_cam":        /* Media print - photos */
        case "charge_print":  /* charge print */
        case "delayed_print": /* delayed print */
        case "private_print": /* private print */
        case "secure_print":  /* secure print */
        case "sample_print":  /* sample print */
        case "jobmem":        /* stored programming */
        case "jobflow":       /* job flow sheets */
        case "bm_scan_s":     /* BMLinkS scan */
        case "bm_pl_p":       /* BMLinkS print */
        case "bm_menu":       /* BMLinkS menu */
        case "web":           /* Web application */
        case "web1" :         /* Web application 1 */
        case "web2" :         /* Web application 2 */
        case "web3" :         /* Web application 3 */
        case "web4" :         /* Web application 4 */
        case "web5" :         /* Web application 5 */
        case "web6" :         /* Web application 6 */
        case "web7" :         /* Web application 7 */
        case "web8" :         /* Web application 8 */
        case "web9" :         /* Web application 9 */
        case "web10":         /* Web application 10*/
        case "cstm1":         /* Custom service 1 */
        case "cstm2":         /* Custom service 2 */
        case "cstm3":         /* Custom service 3 */
        case "cstm4":         /* Custom service 4 */
        case "cstm5":         /* Custom service 5 */
        case "cstm6":         /* Custom service 6 */
        case "cstm7":         /* Custom service 7 */
        case "cstm8":         /* Custom service 8 */
        case "cstm9":         /* Custom service 9 */
        {
          return screenName;
        }

        default:
          // default for all other values
          return "allservice";
      }
    },

    /**
     * Specifies how often to perform connection checks while online
     */
    setOnlineCheck: function(timeout) {
      onlineCheck = timeout;
    },

    /**
     * Specifies how often to perform connection checks while offline
     */
    setOfflineCheck: function(timeout) {
      offlineCheck = timeout;
    },

    isApeosPort4: function() {
      return /.*ApeosPort.IV.*/i.test(deviceModel);
    },

    isApeosPort5: function() {
      return /.*ApeosPort.V.*/i.test(deviceModel);
    }
  };
}();
